@@ -8,6 +8,8 @@ ARCH="s390:31-bit"
 MACHINE=
 NOP=0x07070707
 TEMPLATE_NAME=elf32
-GENERATE_SHLIB_SCRIPT=yes 
-GENERATE_PIE_SCRIPT=yes 
+GENERATE_SHLIB_SCRIPT=yes
+GENERATE_PIE_SCRIPT=yes
 NO_SMALL_DATA=yes
+SEPARATE_GOTPLT=12
+IREL_IN_PLT=
